#######################################
# Order of execution of various modules
#######################################

set ExecutionPath {
  ParticlePropagator
  TrackEfficiency
  TrackMomentumSmearing
  ElectronEnergySmearing
  Calorimeter
  PhotonEfficiency
  PhotonIsolation
  ElectronEfficiency
  ElectronIsolation
  MuonEfficiency
  MuonIsolation
  MissingET
  ParticleFlow
  FastJetFinder
  BTagging
  TauTagging
  UniqueObjectFinder
  TreeWriter
}

#################################
# Propagate particles in cylinder
#################################

module ParticlePropagator ParticlePropagator {
  set InputArray Delphes/candidates

  set OutputArray candidates
  set TrackOutputArray tracks

  # radius of the magnetic field coverage, in m
  set Radius 1.29
  # half-length of the magnetic field coverage, in m
  set HalfLength 3.00

  # magnetic field
  set Bz 3.8
}

##################
# Track efficiency
##################

module Efficiency TrackEfficiency {
  set InputArray ParticlePropagator/tracks
  set OutputArray tracks

  # add EfficiencyFormula {abs(PDG code)} {efficiency formula as a function of eta and pt}

  # default efficiency formula
  add EfficiencyFormula {0} {                                                     (pt <= 0.1)   * (0.000) + \
                                                (abs(eta) <= 1.5) * (pt > 0.1   && pt <= 1.0)   * (0.700) + \
                                                (abs(eta) <= 1.5) * (pt > 1.0)                  * (0.950) + \
                              (abs(eta) > 1.5 && abs(eta) <= 2.5) * (pt > 0.1   && pt <= 1.0)   * (0.600) + \
                              (abs(eta) > 1.5 && abs(eta) <= 2.5) * (pt > 1.0)                  * (0.850) + \
                              (abs(eta) > 2.5)                                                  * (0.000)}

                     
  # efficiency formula for electrons
  add EfficiencyFormula {11} {                                                    (pt <= 0.1)   * (0.000) + \
                                                (abs(eta) <= 1.5) * (pt > 0.1   && pt <= 1.0)   * (0.725) + \
                                                (abs(eta) <= 1.5) * (pt > 1.0   && pt <= 1.0e2) * (0.950) + \
                                                (abs(eta) <= 1.5) * (pt > 1.0e2)                * (0.990) + \
                              (abs(eta) > 1.5 && abs(eta) <= 2.5) * (pt > 0.1   && pt <= 1.0)   * (0.500) + \
                              (abs(eta) > 1.5 && abs(eta) <= 2.5) * (pt > 1.0   && pt <= 1.0e2) * (0.825) + \
                              (abs(eta) > 1.5 && abs(eta) <= 2.5) * (pt > 1.0e2)                * (0.900) + \
                              (abs(eta) > 2.5)                                                  * (0.000)}

  # efficiency formula for muons
  add EfficiencyFormula {13} {                                                    (pt <= 0.1)   * (0.000) + \
                                                (abs(eta) <= 1.5) * (pt > 0.1   && pt <= 1.0)   * (0.750) + \
                                                (abs(eta) <= 1.5) * (pt > 1.0)                  * (1.000) + \
                              (abs(eta) > 1.5 && abs(eta) <= 2.5) * (pt > 0.1   && pt <= 1.0)   * (0.700) + \
                              (abs(eta) > 1.5 && abs(eta) <= 2.5) * (pt > 1.0)                  * (0.975) + \
                              (abs(eta) > 2.5)                                                  * (0.000)}
}

##################
# Track resolution
##################

module MomentumSmearing TrackMomentumSmearing {
  set InputArray TrackEfficiency/tracks
  set OutputArray tracks

  # add ResolutionFormula {abs(PDG code)} {resolution formula as a function of eta and pt}

  # default resolution formula
  add ResolutionFormula {0} {                   (abs(eta) <= 1.5) * (pt > 0.1   && pt <= 1.0)   * (0.020) + \
                                                (abs(eta) <= 1.5) * (pt > 1.0   && pt <= 1.0e1) * (0.012) + \
                                                (abs(eta) <= 1.5) * (pt > 1.0e1 && pt <= 2.0e2) * (0.025) + \
                                                (abs(eta) <= 1.5) * (pt > 2.0e2)                * (0.050) + \
                              (abs(eta) > 1.5 && abs(eta) <= 2.5) * (pt > 0.1   && pt <= 1.0)   * (0.030) + \
                              (abs(eta) > 1.5 && abs(eta) <= 2.5) * (pt > 1.0   && pt <= 1.0e1) * (0.022) + \
                              (abs(eta) > 1.5 && abs(eta) <= 2.5) * (pt > 1.0e1 && pt <= 2.0e2) * (0.035) + \
                              (abs(eta) > 1.5 && abs(eta) <= 2.5) * (pt > 2.0e2)                * (0.050)}
                                   
  # resolution formula for electrons
  add ResolutionFormula {11} {                  (abs(eta) <= 1.5) * (pt > 0.1   && pt <= 1.0)   * (0.015) + \
                                                (abs(eta) <= 1.5) * (pt > 1.0   && pt <= 1.0e1) * (0.009) + \
                                                (abs(eta) <= 1.5) * (pt > 1.0e1 && pt <= 2.0e2) * (0.020) + \
                                                (abs(eta) <= 1.5) * (pt > 2.0e2)                * (0.050) + \
                              (abs(eta) > 1.5 && abs(eta) <= 2.5) * (pt > 0.1   && pt <= 1.0)   * (0.025) + \
                              (abs(eta) > 1.5 && abs(eta) <= 2.5) * (pt > 1.0   && pt <= 1.0e1) * (0.030) + \
                              (abs(eta) > 1.5 && abs(eta) <= 2.5) * (pt > 1.0e1 && pt <= 2.0e2) * (0.030) + \
                              (abs(eta) > 1.5 && abs(eta) <= 2.5) * (pt > 2.0e2)                * (0.050)}

  # resolution formula for muons
  add ResolutionFormula {13} {                  (abs(eta) <= 1.5) * (pt > 0.1   && pt <= 1.0)   * (0.025) + \
                                                (abs(eta) <= 1.5) * (pt > 1.0   && pt <= 1.0e1) * (0.015) + \
                                                (abs(eta) <= 1.5) * (pt > 1.0e1 && pt <= 2.0e2) * (0.030) + \
                                                (abs(eta) <= 1.5) * (pt > 2.0e2)                * (0.050) + \
                              (abs(eta) > 1.5 && abs(eta) <= 2.5) * (pt > 0.1   && pt <= 1.0)   * (0.035) + \
                              (abs(eta) > 1.5 && abs(eta) <= 2.5) * (pt > 1.0   && pt <= 1.0e1) * (0.024) + \
                              (abs(eta) > 1.5 && abs(eta) <= 2.5) * (pt > 1.0e1 && pt <= 2.0e2) * (0.040) + \
                              (abs(eta) > 1.5 && abs(eta) <= 2.5) * (pt > 2.0e2)                * (0.050)}
}

#####################
# Electron resolution
#####################

module EnergySmearing ElectronEnergySmearing {
  set InputArray TrackEfficiency/tracks
  set OutputArray electrons

  # add ResolutionFormula {abs(PDG code)} {resolution formula as a function of eta and energy}

  add ResolutionFormula {11} {                  (abs(eta) <= 1.7) * sqrt(energy^2*0.0055^2 + energy*0.05^2 + 0.25^2) + \
                              (abs(eta) > 1.7 && abs(eta) <= 3.0) * sqrt(energy^2*0.0055^2 + energy*0.05^2 + 0.25^2) + \
                              (abs(eta) > 3.0 && abs(eta) <= 5.0) * sqrt(energy^2*0.1070^2 + energy*2.08^2)}

}

#############
# Calorimeter
#############

module Calorimeter Calorimeter {
  set InputArray ParticlePropagator/candidates

  set TowerOutputArray towers
  set NeutralOutputArray neutrals
  set PhotonOutputArray photons

  set pi [expr {acos(-1)}]
  
  # 5 degrees towers
  set PhiBins 0.0
  for {set i 1} {$i <= 36} {incr i} { 
    add PhiBins [expr {+$i * $pi/36.0}]
    add PhiBins [expr {-$i * $pi/36.0}]
  }
  add EtaPhiBins 0.0 $PhiBins
  for {set i 1} {$i <= 20} {incr i} { 
    add EtaPhiBins [expr {+$i * 0.087}] $PhiBins
    add EtaPhiBins [expr {-$i * 0.087}] $PhiBins
  }

  # 10 degrees towers
  set PhiBins 0.0
  for {set i 1} {$i <= 18} {incr i} { 
    add PhiBins [expr {+$i * $pi/18.0}]
    add PhiBins [expr {-$i * $pi/18.0}]
  }
  foreach eta {1.740 1.830 1.930 2.043 2.172 2.322 2.500 2.650 2.868 2.950 3.125 3.300 3.475 3.650 3.825 4.000 4.175 4.350 4.525} { 
    add EtaPhiBins -$eta $PhiBins
    add EtaPhiBins +$eta $PhiBins
  }
 
  # 20 degrees towers
  set PhiBins 0.0
  for {set i 1} {$i <= 9} {incr i} { 
    add PhiBins [expr {+$i * $pi/9.0}]
    add PhiBins [expr {-$i * $pi/9.0}]
  }  
  foreach eta {4.700 5.000} { 
    add EtaPhiBins -$eta $PhiBins
    add EtaPhiBins +$eta $PhiBins
  }

  # default energy fractions {abs(PDG code)} {Fecal Fhcal}
  add EnergyFraction {0} {0.0 1.0}
  # energy fractions for e, gamma and pi0
  add EnergyFraction {11} {1.0 0.0}
  add EnergyFraction {22} {1.0 0.0}
  add EnergyFraction {111} {1.0 0.0}
  # energy fractions for muon and neutrinos
  add EnergyFraction {12} {0.0 0.0}
  add EnergyFraction {13} {0.0 0.0}
  add EnergyFraction {14} {0.0 0.0}
  add EnergyFraction {16} {0.0 0.0}
  # energy fractions for K0short and Lambda
  add EnergyFraction {310} {0.3 0.7}
  add EnergyFraction {3122} {0.3 0.7}
  
  # set ECalResolutionFormula {resolution formula as a function of eta and energy}
  set ECalResolutionFormula {                  (abs(eta) <= 1.7) * sqrt(energy^2*0.0055^2 + energy*0.05^2 + 0.25^2) + \
                             (abs(eta) > 1.7 && abs(eta) <= 3.0) * sqrt(energy^2*0.0055^2 + energy*0.05^2 + 0.25^2) + \
                             (abs(eta) > 3.0 && abs(eta) <= 5.0) * sqrt(energy^2*0.1070^2 + energy*2.08^2)}

  # set HCalResolutionFormula {resolution formula as a function of eta and energy}
  set HCalResolutionFormula {                  (abs(eta) <= 1.7) * sqrt(energy^2*0.0500^2 + energy*1.50^2) + \
                             (abs(eta) > 1.7 && abs(eta) <= 3.0) * sqrt(energy^2*0.0500^2 + energy*0.50^2) + \
                             (abs(eta) > 3.0 && abs(eta) <= 5.0) * sqrt(energy^2*0.1300^2 + energy*2.70^2)}
}

###################
# Photon efficiency
###################

module Efficiency PhotonEfficiency {
  set InputArray Calorimeter/photons
  set OutputArray photons

  # add EfficiencyFormula {abs(PDG code)} {efficiency formula as a function of eta and pt}

  # default efficiency formula
  add EfficiencyFormula {0} {                                       (pt <= 5.0) * (0.000) + \
                                                (abs(eta) <= 1.5) * (pt > 5.0)  * (0.950) + \
                              (abs(eta) > 1.5 && abs(eta) <= 2.5) * (pt > 5.0)  * (0.850) + \
                              (abs(eta) > 2.5)                                  * (0.000)}
}

##################
# Photon isolation
##################

module Isolation PhotonIsolation {
  set CandidateInputArray PhotonEfficiency/photons
  set IsolationInputArray TrackMomentumSmearing/tracks
  
  set OutputArray photons

  set DeltaRMax 0.5

  set PTMin 0.5

  set PTRatioMax 0.1
}

#####################
# Electron efficiency
#####################

module Efficiency ElectronEfficiency {
  set InputArray ElectronEnergySmearing/electrons
  set OutputArray electrons

  # add EfficiencyFormula {abs(PDG code)} {efficiency formula as a function of eta and pt}

  # default efficiency formula
  add EfficiencyFormula {0} {0.0}

  # efficiency formula for electrons
  add EfficiencyFormula {11} {                                      (pt <= 5.0) * (0.000) + \
                                                (abs(eta) <= 1.5) * (pt > 5.0)  * (0.950) + \
                              (abs(eta) > 1.5 && abs(eta) <= 2.5) * (pt > 5.0)  * (0.850) + \
                              (abs(eta) > 2.5)                                  * (0.000)}
}

####################
# Electron isolation
####################

module Isolation ElectronIsolation {
  set CandidateInputArray ElectronEfficiency/electrons
  set IsolationInputArray TrackMomentumSmearing/tracks
  
  set OutputArray electrons

  set DeltaRMax 0.5

  set PTMin 0.5

  set PTRatioMin 0.9
  set PTRatioMax 1.1
}

#################
# Muon efficiency
#################

module Efficiency MuonEfficiency {
  set InputArray TrackMomentumSmearing/tracks
  set OutputArray muons

  # add EfficiencyFormula {abs(PDG code)} {efficiency as a function of eta and pt}

  # default efficiency formula
  add EfficiencyFormula {0} {0.0}

  # efficiency formula for muons
  add EfficiencyFormula {13} {                                      (pt <= 5.0) * (0.000) + \
                                                (abs(eta) <= 1.5) * (pt > 5.0)  * (0.950) + \
                              (abs(eta) > 1.5 && abs(eta) <= 2.5) * (pt > 5.0)  * (0.850) + \
                              (abs(eta) > 2.5)                                  * (0.000)}
}

################
# Muon isolation
################

module Isolation MuonIsolation {
  set CandidateInputArray MuonEfficiency/muons
  set IsolationInputArray TrackMomentumSmearing/tracks
  
  set OutputArray muons

  set DeltaRMax 0.5

  set PTMin 0.5

  set PTRatioMin 0.9
  set PTRatioMax 1.1
}

###################
# Missing ET merger
###################

module Merger MissingET {
# add InputArray InputArray {filter formula as a function of eta and pt}
  add InputArray MuonEfficiency/muons {1}
  add InputArray Calorimeter/towers {1}
  set MomentumOutputArray momentum
}

######################
# Particle flow merger
######################

module Merger ParticleFlow {
# add InputArray InputArray {filter formula as a function of eta and pt}
  add InputArray TrackMomentumSmearing/tracks {abs(eta) <= 2.5}
  add InputArray Calorimeter/neutrals {abs(eta) <= 2.5}
  add InputArray Calorimeter/towers {abs(eta) > 2.5}
  set OutputArray candidates
}

############
# Jet finder
############

module FastJetFinder FastJetFinder {
  set InputArray ParticleFlow/candidates
  set OutputArray jets

  # algorithm: 1 CDFJetClu, 2 MidPoint, 3 SIScone, 4 kt, 5 Cambridge/Aachen, 6 antikt
  set JetAlgorithm 6
  set ParameterR 1.0

  set ConeRadius 0.5
  set SeedThreshold 1.0
  set ConeAreaFraction 1.0
  set AdjacencyCut 2.0
  set OverlapThreshold 0.75

  set MaxIterations 100
  set MaxPairSize 2
  set Iratch 1
  
  set JetPTMin 10.0
}

###########
# b-tagging
###########

module BTagging BTagging {
  set PartonInputArray Delphes/partons
  set JetInputArray FastJetFinder/jets

  set DeltaR 0.5

  set PartonPTMin 1.0

  set PartonEtaMax 2.5

  # add EfficiencyFormula {abs(PDG code)} {efficiency formula as a function of eta and pt}
  # PDG code = the highest PDG code of a quark or gluon inside DeltaR cone around jet axis
  # gluon's PDG code has the lowest priority

  # default efficiency formula (misidentification rate)
  add EfficiencyFormula {0} {0.001}
  # efficiency formula for c-jets (misidentification rate)
  add EfficiencyFormula {4} {0.1}
  # efficiency formula for b-jets
  add EfficiencyFormula {5} {0.4}
}

module TauTagging TauTagging {
  set ParticleInputArray Delphes/particles
  set PartonInputArray Delphes/partons
  set JetInputArray FastJetFinder/jets

  set OutputArray taus

  set DeltaR 0.5

  set TauPTMin 1.0

  set TauEtaMax 2.5

  # add EfficiencyFormula {abs(PDG code)} {efficiency formula as a function of eta and pt}

  # default efficiency formula (misidentification rate)
  add EfficiencyFormula {0} {0.001}
  # efficiency formula for tau-jets
  add EfficiencyFormula {15} {0.4}
}

#####################################################
# Find uniquely identified photons/electrons/tau/jets
#####################################################

module UniqueObjectFinder UniqueObjectFinder {
# earlier arrays take precedence over later ones
# add InputArray InputArray OutputArray
  add InputArray PhotonIsolation/photons photons
  add InputArray ElectronIsolation/electrons electrons
  add InputArray TauTagging/taus taus
  add InputArray FastJetFinder/jets jets
}

##################
# ROOT tree writer
##################

module TreeWriter TreeWriter {
# add Branch InputArray BranchName BranchClass
  add Branch Delphes/particles Particle GenParticle
  add Branch TrackMomentumSmearing/tracks Track Track
  add Branch Calorimeter/towers Tower Tower
  add Branch UniqueObjectFinder/photons Photon Photon
  add Branch UniqueObjectFinder/electrons Electron Electron
  add Branch UniqueObjectFinder/taus TauJet TauJet
  add Branch UniqueObjectFinder/jets Jet Jet
  add Branch MuonIsolation/muons Muon Muon
  add Branch MissingET/momentum MissingET MissingET
}
