#ifndef Isolation_h
#define Isolation_h

/** \class Isolation
 *
 *  Sums transverse momenta of isolation objects (tracks, calorimeter towers, etc)
 *  within a DeltaR cone around a candidate and calculates fraction of this sum
 *  to the candidate's transverse momentum. outputs candidates that have
 *  the transverse momenta fraction within (PTRatioMin, PTRatioMax]. 
 *
 *  $Date: 2013-01-10 11:46:21 +0100 (Thu, 10 Jan 2013) $
 *  $Revision: 875 $
 *
 *
 *  \author P. Demin - UCL, Louvain-la-Neuve
 *
 */

#include "classes/DelphesModule.h"

class TObjArray;

class ExRootFilter;
class IsolationClassifier;

class Isolation: public DelphesModule
{
public:

  Isolation();
  ~Isolation();

  void Init();
  void Process();
  void Finish();

private:

  Double_t fDeltaRMin, fDeltaRMax;
  
  Double_t fPTRatioMin, fPTRatioMax;

  IsolationClassifier *fClassifier; //!
  
  ExRootFilter *fFilter;

  TIterator *fItIsolationInputArray; //!
  
  TIterator *fItCandidateInputArray; //!

  const TObjArray *fIsolationInputArray; //!
  
  const TObjArray *fCandidateInputArray; //!

  TObjArray *fOutputArray; //!

  ClassDef(Isolation, 1)
};

#endif
