#ifndef ParticlePropagator_h
#define ParticlePropagator_h

/** \class ParticlePropagator
 *
 *  Propagates charged and neutral particles
 *  from a given vertex to a cylinder defined by its radius, 
 *  its half-length, centered at (0,0,0) and with its axis
 *  oriented along the z-axis.
 *
 *  $Date: 2012-11-18 15:57:08 +0100 (Sun, 18 Nov 2012) $
 *  $Revision: 814 $
 *
 *
 *  \author P. Demin - UCL, Louvain-la-Neuve
 *
 */

#include "classes/DelphesModule.h"

class TClonesArray;
class TIterator;

class ParticlePropagator: public DelphesModule
{
public:

  ParticlePropagator();
  ~ParticlePropagator();

  void Init();
  void Process();
  void Finish();

private:

  Double_t fRadius, fRadius2, fHalfLength;
  Double_t fBz;

  TIterator *fItInputArray; //!

  const TObjArray *fInputArray; //!

  TObjArray *fOutputArray; //!
  TObjArray *fTrackOutputArray; //!

  ClassDef(ParticlePropagator, 1)
};

#endif
