#ifndef TreeWriter_h
#define TreeWriter_h

/** \class TreeWriter
 *
 *  Fills ROOT tree branches.
 *
 *  $Date: 2012-11-18 15:57:08 +0100 (Sun, 18 Nov 2012) $
 *  $Revision: 814 $
 *
 *
 *  \author P. Demin - UCL, Louvain-la-Neuve
 *
 */

#include "classes/DelphesModule.h"

#include <map>

class TClass;
class TIterator;
class ExRootTreeBranch;

class TreeWriter: public DelphesModule
{
public:

  TreeWriter();
  ~TreeWriter();

  void Init();
  void Process();
  void Finish();

private:

  void ProcessParticles(ExRootTreeBranch *branch, TIterator *iterator);
  void ProcessTracks(ExRootTreeBranch *branch, TIterator *iterator);
  void ProcessTowers(ExRootTreeBranch *branch, TIterator *iterator);
  void ProcessPhotons(ExRootTreeBranch *branch, TIterator *iterator);
  void ProcessElectrons(ExRootTreeBranch *branch, TIterator *iterator);
  void ProcessMuons(ExRootTreeBranch *branch, TIterator *iterator);
  void ProcessTauJets(ExRootTreeBranch *branch, TIterator *iterator);
  void ProcessJets(ExRootTreeBranch *branch, TIterator *iterator);
  void ProcessMissingET(ExRootTreeBranch *branch, TIterator *iterator);

#ifndef __CINT__
  typedef void (TreeWriter::*TProcessMethod)(ExRootTreeBranch *, TIterator *); //!

  typedef std::map< ExRootTreeBranch *, std::pair< TProcessMethod, TIterator * > > TBranchMap; //!

  TBranchMap fBranchMap; //!

  std::map< TClass *, TProcessMethod > fClassMap; //!
#endif
  ClassDef(TreeWriter, 1)
};

#endif
